"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArkaneSubProvider = void 0;
var tslib_1 = require("tslib");
var arkane_connect_1 = require("@arkane-network/arkane-connect");
var base_wallet_subprovider_1 = require("@0x/subproviders/lib/src/subproviders/base_wallet_subprovider");
var ArkaneSubProvider = /** @class */ (function (_super) {
    tslib_1.__extends(ArkaneSubProvider, _super);
    function ArkaneSubProvider(options) {
        var _this = _super.call(this) || this;
        _this.wallets = [];
        _this.authenticated = false;
        var connectConstructorOptions = {
            environment: options.environment || 'production',
            bearerTokenProvider: options.bearerTokenProvider,
        };
        if (options.signMethod) {
            Object.assign(connectConstructorOptions, { signUsing: options.signMethod == 'POPUP' ? arkane_connect_1.SignMethod.POPUP : arkane_connect_1.SignMethod.REDIRECT });
        }
        if (options.windowMode) {
            Object.assign(connectConstructorOptions, { windowMode: options.windowMode == 'POPUP' ? arkane_connect_1.WindowMode.POPUP : arkane_connect_1.WindowMode.REDIRECT });
        }
        _this.arkaneConnect = new arkane_connect_1.ArkaneConnect(options.clientId, connectConstructorOptions);
        _this.network = options.network;
        _this.options = options;
        return _this;
    }
    ArkaneSubProvider.prototype.startGetAccountFlow = function (authenticationOptions) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var that;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                if (authenticationOptions) {
                    this.options.authenticationOptions = authenticationOptions;
                }
                that = this;
                return [2 /*return*/, this.arkaneConnect.flows.getAccount(arkane_connect_1.SecretType.ETHEREUM, this.options.authenticationOptions)
                        .then(function (account) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, new Promise(function (resolve, reject) {
                                        if (!account.isAuthenticated) {
                                            console.debug('Not authenticated to Arkane Network');
                                            reject('not-authenticated');
                                        }
                                        else if (account.wallets && account.wallets.length <= 0) {
                                            console.debug('No wallet has been linked to this application');
                                            reject('no-wallet-linked');
                                        }
                                        else {
                                            console.debug("Authenticated to Arkane Network and at least one wallet is linked to this application");
                                            that.authenticated = true;
                                            that.wallets = account.wallets;
                                            that.lastWalletsFetch = Date.now();
                                            resolve(account);
                                        }
                                    })];
                                case 1: return [2 /*return*/, _a.sent()];
                            }
                        });
                    }); })];
            });
        });
    };
    ArkaneSubProvider.prototype.refreshWalletsFromApi = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var that;
            return tslib_1.__generator(this, function (_a) {
                that = this;
                return [2 /*return*/, this.arkaneConnect.api.getWallets({ secretType: arkane_connect_1.SecretType.ETHEREUM, includeBalance: false })
                        .then(function (returnedWallets) {
                        that.wallets = returnedWallets;
                    })];
            });
        });
    };
    /**
     * Retrieve the accounts associated with the eth-lightwallet instance.
     * This method is implicitly called when issuing a `eth_accounts` JSON RPC request
     * via your providerEngine instance.
     *
     * @return An array of accounts
     */
    ArkaneSubProvider.prototype.getAccountsAsync = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var that, promise;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                that = this;
                if (!this.authenticated) {
                    promise = this.startGetAccountFlow();
                }
                else if (this.shouldRefreshWallets()) {
                    this.lastWalletsFetch = Date.now();
                    promise = this.refreshWalletsFromApi();
                }
                else {
                    promise = Promise.resolve();
                }
                return [2 /*return*/, promise.then(function () {
                        return _this.wallets.map(function (wallet) { return wallet.address; });
                    })];
            });
        });
    };
    ArkaneSubProvider.prototype.shouldRefreshWallets = function () {
        return !this.lastWalletsFetch
            || (Date.now() - this.lastWalletsFetch) > 5000;
    };
    ArkaneSubProvider.prototype.checkAuthenticated = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, this.arkaneConnect.checkAuthenticated().then(function (authResult) {
                        _this.authenticated = authResult.isAuthenticated;
                        return authResult;
                    })];
            });
        });
    };
    /**
     * Signs a transaction with the account specificed by the `from` field in txParams.
     * If you've added this Subprovider to your app's provider, you can simply send
     * an `eth_sendTransaction` JSON RPC request, and this method will be called auto-magically.
     * If you are not using this via a ProviderEngine instance, you can call it directly.
     * @param txParams Parameters of the transaction to sign
     * @return Signed transaction hex string
     */
    ArkaneSubProvider.prototype.signTransactionAsync = function (txParams) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var signer;
            return tslib_1.__generator(this, function (_a) {
                signer = this.arkaneConnect.createSigner();
                return [2 /*return*/, signer.signTransaction(this.constructEthereumTransationSignatureRequest(txParams))
                        .then(function (result) {
                        if (result.status === 'SUCCESS') {
                            return result.result.signedTransaction;
                        }
                        else {
                            throw new Error((result.errors && result.errors.join(", ")));
                        }
                    })];
            });
        });
    };
    ArkaneSubProvider.prototype.constructEthereumTransationSignatureRequest = function (txParams) {
        console.debug(txParams);
        var retVal = {
            gasPrice: txParams.gasPrice ? parseInt(txParams.gasPrice, 16) : txParams.gasPrice,
            gas: txParams.gas ? parseInt(txParams.gas, 16) : txParams.gas,
            to: txParams.to,
            nonce: txParams.nonce ? parseInt(txParams.nonce, 16) : txParams.nonce,
            data: (txParams.data) || '0x',
            value: txParams.value ? parseInt(txParams.value, 16) : 0,
            submit: false,
            type: arkane_connect_1.SignatureRequestType.ETHEREUM_TRANSACTION,
            walletId: this.getWalletIdFrom(txParams.from),
            network: this.network
        };
        return retVal;
    };
    /**
     * Sign a personal Ethereum signed message. The signing account will be the account
     * associated with the provided address.
     * If you've added this Subprovider to your app's provider, you can simply send an `eth_sign`
     * or `personal_sign` JSON RPC request, and this method will be called auto-magically.
     * If you are not using this via a ProviderEngine instance, you can call it directly.
     * @param data Hex string message to sign
     * @param address Address of the account to sign with
     * @return Signature hex string (order: rsv)
     */
    ArkaneSubProvider.prototype.signPersonalMessageAsync = function (data, address) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var signer;
            return tslib_1.__generator(this, function (_a) {
                signer = this.arkaneConnect.createSigner();
                return [2 /*return*/, signer.signTransaction({
                        type: arkane_connect_1.SignatureRequestType.ETHEREUM_RAW,
                        walletId: this.getWalletIdFrom(address),
                        data: data
                    })
                        .then(function (result) {
                        if (result.status === 'SUCCESS') {
                            return result.result.signature;
                        }
                        else {
                            throw new Error((result.errors && result.errors.join(", ")));
                        }
                    })];
            });
        });
    };
    /**
     * Sign an EIP712 Typed Data message. The signing address will associated with the provided address.
     * If you've added this Subprovider to your app's provider, you can simply send an `eth_signTypedData`
     * JSON RPC request, and this method will be called auto-magically.
     * If you are not using this via a ProviderEngine instance, you can call it directly.
     * @param address Address of the account to sign with
     * @param data the typed data object
     * @return Signature hex string (order: rsv)
     */
    ArkaneSubProvider.prototype.signTypedDataAsync = function (address, typedData) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var signer, request;
            return tslib_1.__generator(this, function (_a) {
                signer = this.arkaneConnect.createSigner();
                if (typeof typedData === 'string') {
                    typedData = JSON.parse(typedData);
                }
                request = {
                    data: typedData,
                    walletId: this.getWalletIdFrom(address),
                    secretType: arkane_connect_1.SecretType.ETHEREUM
                };
                return [2 /*return*/, signer.signEip712(request)
                        .then(function (result) {
                        if (result.status === 'SUCCESS') {
                            return result.result.signature;
                        }
                        else {
                            throw new Error((result.errors && result.errors.join(", ")));
                        }
                    })];
            });
        });
    };
    ArkaneSubProvider.prototype.getWalletIdFrom = function (address) {
        var foundWallet = this.wallets.find(function (wallet) {
            return wallet.address.toLowerCase() === address.toLowerCase();
        });
        return (foundWallet && foundWallet.id) || '';
    };
    return ArkaneSubProvider;
}(base_wallet_subprovider_1.BaseWalletSubprovider));
exports.ArkaneSubProvider = ArkaneSubProvider;
//# sourceMappingURL=ArkaneSubProvider.js.map