"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ArkaneSubProvider_1 = require("./ArkaneSubProvider");
var NonceTracker_1 = require("./NonceTracker");
var SignedVersionedTypedDataSubProvider_1 = require("./SignedVersionedTypedDataSubProvider");
var ProviderEngine = require('web3-provider-engine');
var CacheSubprovider = require('web3-provider-engine/subproviders/cache');
var FixtureSubprovider = require('web3-provider-engine/subproviders/fixture');
var FilterSubprovider = require('web3-provider-engine/subproviders/filters');
var RpcSubprovider = require('web3-provider-engine/subproviders/rpc');
var Arkane = /** @class */ (function () {
    function Arkane() {
    }
    Arkane.prototype.arkaneConnect = function (network) {
        this.network = network;
        this.originalNetwork = network;
        return this.ac;
    };
    Arkane.prototype.changeNetwork = function (network) {
        if (network && network.nodeUrl) {
            this.network = network;
            this.nonceSubProvider.rpcUrl = network.nodeUrl;
            this.rpcSubprovider.rpcUrl = network.nodeUrl;
            this.arkaneSubProvider.network = network;
        }
        else {
            console.warn("Not changing to network, not sufficient data: resetting network", network);
            this.resetNetwork();
        }
    };
    Arkane.prototype.checkAuthenticated = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (!this.arkaneSubProvider) {
                    throw new Error("Please initialise provider first (Arkane.createArkaneProviderEngine)");
                }
                return [2 /*return*/, this.arkaneSubProvider.checkAuthenticated()];
            });
        });
    };
    Arkane.prototype.authenticate = function (authenticationOptions) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (!this.arkaneSubProvider) {
                    throw new Error("Please initialise provider first (Arkane.createArkaneProviderEngine)");
                }
                return [2 /*return*/, this.arkaneSubProvider.startGetAccountFlow(authenticationOptions)];
            });
        });
    };
    Arkane.prototype.resetNetwork = function () {
        this.network = this.originalNetwork;
    };
    Arkane.prototype.createArkaneProviderEngine = function (options) {
        var _this = this;
        var engine = new ProviderEngine();
        engine.addProvider(new FixtureSubprovider({
            web3_clientVersion: 'ArkaneProviderEngine/v0.0.1/javascript',
            net_listening: true,
            eth_hashrate: '0x00',
            eth_mining: false,
            eth_syncing: true,
        }));
        engine.addProvider(new CacheSubprovider());
        engine.addProvider(new FilterSubprovider());
        var endpoint = (options.rpcUrl || (options.network ? options.network.nodeUrl : undefined)) || this.getDefaultEndpoint(options);
        this.nonceSubProvider = new NonceTracker_1.NonceTrackerSubprovider({ rpcUrl: endpoint });
        engine.addProvider(this.nonceSubProvider);
        this.arkaneSubProvider = new ArkaneSubProvider_1.ArkaneSubProvider(options);
        this.ac = this.arkaneSubProvider.arkaneConnect;
        this.rpcSubprovider = new RpcSubprovider({ rpcUrl: endpoint });
        this.signedVersionedTypedDataSubProvider = new SignedVersionedTypedDataSubProvider_1.SignedVersionedTypedDataSubProvider(this.arkaneSubProvider);
        engine.addProvider(this.signedVersionedTypedDataSubProvider);
        return options.skipAuthentication
            ? Promise.resolve(this.startEngine(engine))
            : this.arkaneSubProvider.getAccountsAsync().then(function () { return _this.startEngine(engine); });
    };
    Arkane.prototype.getDefaultEndpoint = function (options) {
        if (options.environment && (options.environment === 'qa' || options.environment === 'staging')) {
            return 'https://rinkeby.arkane.network';
        }
        return 'https://ethereum.arkane.network';
    };
    Arkane.prototype.startEngine = function (engine) {
        engine.addProvider(this.arkaneSubProvider);
        engine.addProvider(this.rpcSubprovider);
        // network connectivity error
        engine.on('error', function (err) {
            // report connectivity errors
            console.error(err.stack);
        });
        // start polling for blocks
        engine.start();
        return engine;
    };
    return Arkane;
}());
exports.default = Arkane;
if (typeof window !== 'undefined') {
    window.Arkane = new Arkane();
}
//# sourceMappingURL=index.js.map